#include "SolderingIronController.h"

#define DS18B20_IO PIN_A5 
//#define DS18B20_PU PIN_B1 
#include "ds18b20.c"

volatile signed int16 br;
volatile signed int16 br_min;
volatile signed int16 br_max;
volatile unsigned int8 br_increment;
volatile unsigned int8 f;
volatile unsigned int8 triggered;

volatile unsigned int16 temperature; // 0-500 Target temperature in 0.1C
volatile unsigned int16 current;     // 0-1024 Current temperature in 0.1C
volatile signed int16 ambient;       // Ambient temp in 0.1C
volatile signed int16 offset;        // Offset in 0.1C
volatile float32 slope;              // Slope coeficient

unsigned int8  i, j;
unsigned int8  HYST;
unsigned int8  DS;

unsigned char *p;

#define INCREASEIT() {if (br < br_max) br+=br_increment;}
#define DECREASEIT() {if (br > br_min) br-=br_increment;}

#INT_RA
void  RA_isr(void){
  static unsigned int8 old1=0;
  static unsigned int8 old2=0;
  
  unsigned int8 new1;
  unsigned int8 new2;
  
  new1 = input(ENCODER1);
  new2 = input(ENCODER2);
  
  if (old1 != new1){ //E1 triggered
    if (new1){ // E1 low->high
      if (new2){
        INCREASEIT();
      } else {
        DECREASEIT();
      }
    } else {   // E1 high->low
      if (new2){
        DECREASEIT();
      } else {
        INCREASEIT();
      }    
    }
  } else {           //E2 triggered
    if (new2){ // E2 low->high
      if (new1){
        DECREASEIT();
      } else {
        INCREASEIT();
      }
    } else {   // E2 high->low
      if (new1){
        INCREASEIT();
      } else {
        DECREASEIT();
      }    
    }
  
  } 
  old1 = new1;
  old2 = new2;
  f = BAR_DELAY;
  triggered = 1;
  if (br > br_max) br = br_max;  
}



// get the temp in C from 0-1023 (0-5V)
// temperature of the tip is ambient + k-type response
// 
// k-type table should apply
// 200C 8.138mV
// 210C 8.539mV
// 220C 8.940mV
// 230C 9.343mV
// ...
// 300C 12.209mV
// 310C 12.624mV
// 320C 13.040mV
// ...
// 400C 16.397mV
// ...
// 500C 20.644mV
//
// op-amp gain for the k-type is 240 so the standard
// so the actual temp is ADC/2 in C
unsigned int16 adc2temp (unsigned int16 a){ 
   if(DS){ 
      //We are using DS18B20 so assume input is K-type thermocouple
      return ambient + a*5; // ADC/2 *10 as we return temp in *10
   } else {
      // Not using DS, so input is 240* intput voltage (from voltage divider)
      // this only works if ADC values changes linearly with temperature
      // but mine tests with HAKKO 907 pencil show that it's PTC changes
      // very linearly in the 200C - 400C range and that's the only range
      // we are interested here
      // note that slope is FLOAT so this calculation takes a whole lot
      // of flash. Converting slope to some int16 would be a way to 
      // save on flash
      return slope * a + offset; 
   }
}


void menu(){
  //turn off the heater
  HEATER_OFF();  
   

  br_min = 0;
  br_max = 2;
  br_increment = 1;
  br = 1;
  printf(lcd_putc, "\f      Setup\n       Menu");
  while (input(ENCODERB)) restart_wdt(); // wait for Function key to continue
  while (!input(ENCODERB)) restart_wdt(); // wait for release
  if (br != 1) return;

  // DEFAULT TEMPERATURE
  printf(lcd_putc, "\f     %4Lu\n Power On  TEMP", temperature/10);
  br_min = 0;
  br_max = 500;
  br_increment = 10;
  br = temperature/10;
  if (br > br_max) br = br_max; if (br < br_min) br = br_min;  
  while (input(ENCODERB)){  
     lcd_gotoxy(6,1);
     printf(lcd_putc, "%4Ld", br );
  }
  temperature = br*10;
  
  while (!input(ENCODERB)) restart_wdt(); // wait for release
  p = (unsigned char *) &temperature;
  write_eeprom(0, *p++);
  write_eeprom(1, *p);


  // DEFAULT HYSTERESIS
  printf(lcd_putc, "\f     %4u\n   HYSTERESIS", HYST);
  br_min = 1;
  br_max = 20;
  br_increment = 1;
  br = HYST;  
  if (br > br_max) br = br_max; if (br < br_min) br = br_min;
  while (input(ENCODERB)){
     lcd_gotoxy(6,1);
     printf(lcd_putc, "%4u", (unsigned int8) br & 0xff);
  }
  while (!input(ENCODERB)) restart_wdt(); // wait for release
  HYST = (br * 10) & 0xff;
  write_eeprom(3, HYST);
  
  
  // DEFAULT USE DS OR NOT
  if (DS){
    printf(lcd_putc, "\f       YES\n   USE DS18B20");
  } else {
    printf(lcd_putc, "\f       NO\n   USE DS18B20");  
  }
  br_min = 0;
  br_max = 1;
  br_increment = 1;
  br = DS;
  if (br > br_max) br = br_max; if (br < br_min) br = br_min;
  while (input(ENCODERB)){
    lcd_gotoxy(1,1);
    if (br){
      printf(lcd_putc, "       YES");
    } else {
      printf(lcd_putc, "       NO ");
    }
  }
  while (!input(ENCODERB)) restart_wdt(); // wait for release
  DS = (unsigned int8)  br & 0xff;
  write_eeprom(4, DS);  
  
  if (!DS){ //if not DS then use offset for ambient
    //OFFSET
    printf(lcd_putc, "\f     %4u\n   Offset", HYST);
    br_min = -1000;
    br_max =  1000;
    br_increment = 1;
    br = offset;  
    if (br > br_max) br = br_max; if (br < br_min) br = br_min;
    while (input(ENCODERB)){
      lcd_gotoxy(6,1);
      printf(lcd_putc, "%4Ld", br );
    }
    while (!input(ENCODERB)) restart_wdt(); // wait for release
    offset = br;
    p = (unsigned char *) &offset;
    write_eeprom(5,*p++);
    write_eeprom(6,*p);  
  }
  
  triggered = 0;
  f=0;

}


void main()
{
   unsigned char rising;
   unsigned int16 ADC_RAW;
   //unsigned int16 ADC_MIN;
   //unsigned int16 ADC_MAX;
   //unsigned int32 ADC_OUT;
   
   setup_oscillator(OSC_8MHZ) ;
   setup_adc_ports (sAN0) ;
   setup_adc (ADC_CLOCK_DIV_16) ;
   set_adc_channel(0) ;
   setup_comparator (NC_NC_NC_NC) ;

   delay_ms(100);

   lcd_init();
   HEATER_OFF();
   printf(lcd_putc, "CRSN Lemilo v2.7\nelco.crsndoo.com");
   
   current = 0;
   f = 0;
   triggered = 0;


   p = (unsigned char *) &temperature;
   *p++ = read_eeprom(0);
   *p = read_eeprom(1);   
   if (temperature > 5000){ //startup temp no larger then 500C
      temperature = 2000;
   }

   //DEBOUNCE_TIME = read_eeprom(2);
   
   HYST = read_eeprom(3);
   DS = read_eeprom(4);

   p = (unsigned char *) &offset;
   *p++ = read_eeprom(5);
   *p = read_eeprom(6);
   
   p = (unsigned char *) &slope;
   *p++ = read_eeprom(7 );
   *p++ = read_eeprom(8 );
   *p++ = read_eeprom(9 );
   *p   = read_eeprom(10);
  
   
   /*
   slope = 3.428571429;
   p = (unsigned char *) &slope;
   write_eeprom(7,  *p++);   
   write_eeprom(8,  *p++);   
   write_eeprom(9,  *p++);   
   write_eeprom(10, *p++);   
   offset = -19;
   p = (unsigned char *) &offset;
   write_eeprom(5,  *p++);   
   write_eeprom(6,  *p);   
   */
   

   delay_ms(2000);
   lcd_putc('\f');
   
   //Enable Interrupt ON CHANGE on 2 pins where encoder is connected
   enable_interrupts(INT_RA1|INT_RA2);
   *(0x96) = 6; //the enable interrupts should set this properly but just in case
   enable_interrupts(GLOBAL);

   br = temperature;   
   br_min = 0;
   br_max = 10000;
   br_increment = 10;
   ambient = offset;
   if (DS) ambient = ds18b20_get_temperature();
   f=0;
   rising = 1;

mainloop:
   if (triggered){ 
      triggered = 0;
   } else {
      br = temperature;
      if (DS){
        if (f == 0) ambient = ds18b20_get_temperature();
      } else {
        delay_ms(100);
      }
   }   

   if ( !input(ENCODERB) ){ // button pressed
     while (!input(ENCODERB)) restart_wdt(); // wait for release
     menu();
     br = temperature;
     br_min = 0;
     br_max = 10000;
     br_increment = 10;
     f = 0;
   }
   
   temperature = br; 
   br_increment = 10;
   
   //ADC_MIN = 0xFFFF;
   //ADC_MAX = 0;
   //ADC_OUT = 0;
   //for (i = 0; i<10; i++){
     ADC_RAW = read_adc();
     //if (ADC_MIN > ADC_RAW) ADC_MIN = ADC_RAW;
     //if (ADC_MAX < ADC_RAW) ADC_MAX = ADC_RAW;
     //ADC_OUT += ADC_RAW;
   //}
   
   //ADC_OUT -= ADC_MIN;
   //ADC_OUT -= ADC_MAX;
   //ADC_RAW = (signed int16) ADC_OUT >> 3;
   current = (current*7  + adc2temp(ADC_RAW)) >> 3; 

   if (f == 0){
       lcd_gotoxy(1,1);
       printf(lcd_putc, "Current: %4Lu.%1u ", (unsigned int16) current     / 10, (unsigned int8) current     % 10);
       lcd_gotoxy(1,2);
       printf(lcd_putc, "Target : %4Lu ", (unsigned int16) temperature / 10);
   } else {
       lcd_gotoxy(1,1);
       j = (temperature/10) >> 6; 
       if (j>15) j=15;
       for (i = 0; i < j; ++i){
          lcd_putc('#');
       }
       for (i = j; i < 15; ++i){
          lcd_putc('.');
       }
       lcd_gotoxy(1,2);
       printf(lcd_putc, "Target : %4Lu", (unsigned int16) temperature/10);
   }

   if (rising){
     if (current < temperature){
       HEATER_ON();
     }else if (current > temperature){
       HEATER_OFF();
       rising = 0;
     }   
   } else {
     if (current < temperature - HYST){
       HEATER_ON();
       rising = 1;
     }else if (current > temperature){
       HEATER_OFF();
     }   
   }

   if (f > 0) --f;
    
   goto mainloop;
}


