
int1 ds18b20_initialization(void){ 
   int1 ready; 
   
   output_float(DS18B20_IO); 
   #ifdef DS18B20_PU
     output_low(DS18B20_PU);     
   #endif
   delay_ms(100);
   
   output_low(DS18B20_IO); 
   delay_us(490);          //min 480us 
   output_float(DS18B20_IO); 
   delay_us(75);           //15 - 60us for device to respond 
   ready=!input(DS18B20_IO); 
   delay_us(415); 
   return ready; 
} 

void ds18b20_write_bit(int8 b){ 
   output_low(DS18B20_IO); 
   if(b == 1){ 
      delay_us(5);         //min 1us 
      output_float(DS18B20_IO); 
   } 
   delay_us(65); 
   output_float(DS18B20_IO); 
   delay_us(2);            //min 1us between write slots 
} 

void ds18b20_write_byte(int8 B){ 
   int8 i, aux; 
    
   for(i=0; i<8; i++){ 
      aux = B >> i;     //aux equals B shifted i times to the right 
      aux &= 0x01;      //least significant bit survives 
      ds18b20_write_bit(aux); 
   } 
   //delay_us(120); 
} 

int1 ds18b20_read_bit(void){ 
   output_low(DS18B20_IO); 
   delay_us(3);         //min 1us 
   output_float(DS18B20_IO); 
   delay_us(5); 
   return(input(DS18B20_IO)); //next a delay of 60 to 120us must be done! 
} 

int8 ds18b20_read_byte(void){ 
   int8 i, result=0x00; 
    
   for(i=0; i<8; i++){ 
      if(ds18b20_read_bit()) 
         result |= (0x01 << i); 
      delay_us(65); 
   } 
   return result; 
} 

int1 ds18b20_start_conversion(void){
  //unsigned int8 busy = 0; 
  if (ds18b20_initialization () ){
      ds18b20_write_byte (0xCC); //skip ROM command
      ds18b20_write_byte (0x44); //convert temperature command
      output_float (DS18B20_IO);
      #ifdef DS18B20_PU
        output_high (DS18B20_PU) ;
      #endif
      delay_ms (800);   //max conversion time FOR 12bit resolution = 750ms
      //while (busy == 0) busy = ds18b20_read_byte ();                  
      #ifdef DS18B20_PU
        output_low (DS18B20_PU);
      #endif
  } else {      
      return 0;
  }
  return 1;
}

signed int16 ds18b20_get_temperature(){
      signed int16 temp;
      unsigned int8 h,l;
      signed int16 temperature;
      
      if (!ds18b20_start_conversion()) return -1;
      
      ds18b20_initialization ();
      ds18b20_write_byte (0xCC); //skip ROM command
      ds18b20_write_byte (0xBE); //read scratch pad command
      
      l = ds18b20_read_byte();
      h = ds18b20_read_byte();

      temp = make16 (h, l);
      
      //temperature = temp / 16.0;
      /*
      temperature = (temp >> 4) & 0x00FF;
      IF (temp&0x0001) temperature = temperature + 0.06250;
      IF (temp&0x0002) temperature = temperature + 0.12500;
      IF (temp&0x0004) temperature = temperature + 0.25000;
      IF (temp&0x0008) temperature = temperature + 0.50000;
      */
      temperature = temp * 0.625;
      //printf (lcd_putc, "\f %7.3fC\n%X %X", temperature, h, l);
      //printf (lcd_putc, "\f %LiC\n%X %X", temperature, h, l);
      return temperature;
}
